@@ -4,7 +4,7 @@ use warnings;
 use autodie;
 package CPAN::Meta::Validator;
 BEGIN {
-  $CPAN::Meta::Validator::VERSION = '2.101091';
+  $CPAN::Meta::Validator::VERSION = '2.101110';
 }
 # ABSTRACT: validate CPAN distribution metadata structures
 
@@ -56,14 +56,16 @@ my $no_index_1_1 = {
 };
 
 my $prereq_map = {
-  ':key' => {
-    name => \&phase,
-    'map' => {
-      ':key'  => {
-        name => \&relation,
-        'map' => $module_map1,
+  map => {
+    ':key' => {
+      name => \&phase,
+      'map' => {
+        ':key'  => {
+          name => \&relation,
+          %$module_map1,
+        },
       },
-    },
+    }
   },
 };
 
@@ -92,7 +94,8 @@ my %definitions = (
     'no_index'    => $no_index_1_3,
     'optional_features'   => {
       'map'       => {
-        ':key'  => { name => \&identifier,
+        ':key'  => {
+          name => \&identifier,
           'map'   => {
             description        => { value => \&string },
             prereqs => $prereq_map,
@@ -102,29 +105,40 @@ my %definitions = (
     },
     'prereqs' => $prereq_map,
     'provides'    => {
-      'map'       => { ':key' => { name  => \&module,
-          'map' => { file    => { mandatory => 1, value => \&file },
-            version => { value => \&version } } } }
+      'map'       => {
+        ':key' => {
+          name  => \&module,
+          'map' => {
+            file    => { mandatory => 1, value => \&file },
+            version => { value => \&version }
+          }
+        }
+      }
     },
     'resources'   => {
-      'map'       => { license    => { lazylist => { value => \&url } },
+      'map'       => {
+        license    => { lazylist => { value => \&url } },
         homepage   => { value => \&url },
-        bugtracker => { 'map' => {
+        bugtracker => {
+          'map' => {
             web => { value => \&url },
             mailto => { value => \&string},
-          }},
-        repository => { 'map' => {
+          }
+        },
+        repository => {
+          'map' => {
             web => { value => \&url },
             url => { value => \&url },
             type => { value => \&string },
-          }},
+          }
+        },
         ':key'     => { value => \&string, name => \&custom_2 },
       }
     },
 
     # CUSTOM -- additional user defined key/value pairs
     # note we can only validate the key name, as the structure is user defined
-    ':key'        => { name => \&custom_2 },
+    ':key'        => { name => \&custom_2, value => \&anything },
   },
 
 '1.4' => {
@@ -182,7 +196,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&string },
+  ':key'        => { name => \&string, value => \&anything },
 },
 
 '1.3' => {
@@ -239,7 +253,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&string },
+  ':key'        => { name => \&string, value => \&anything },
 },
 
 # v1.2 is misleading, it seems to assume that a number of fields where created
@@ -300,7 +314,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&string },
+  ':key'        => { name => \&string, value => \&anything },
 },
 
 # note that the 1.1 spec only specifies 'version' as mandatory
@@ -323,7 +337,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&string },
+  ':key'        => { name => \&string, value => \&anything },
 },
 
 # note that the 1.0 spec doesn't specify optional or mandatory fields
@@ -344,7 +358,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&string },
+  ':key'        => { name => \&string, value => \&anything },
 },
 );
 
@@ -384,6 +398,9 @@ sub errors {
 }
 
 
+my $spec_error = "Missing validation action in specification. "
+  . "Must be one of 'map', 'list', 'lazylist', or 'value'";
+
 sub check_map {
     my ($self,$spec,$data) = @_;
 
@@ -416,6 +433,8 @@ sub check_map {
                 $self->check_list($spec->{$key}{'list'},$data->{$key});
             } elsif($spec->{$key}{'lazylist'}) {
                 $self->check_lazylist($spec->{$key}{'lazylist'},$data->{$key});
+            } else {
+                $self->_error( "$spec_error for '$key'" );
             }
 
         } elsif ($spec->{':key'}) {
@@ -428,6 +447,8 @@ sub check_map {
                 $self->check_list($spec->{':key'}{'list'},$data->{$key});
             } elsif($spec->{':key'}{'lazylist'}) {
                 $self->check_lazylist($spec->{':key'}{'lazylist'},$data->{$key});
+            } else {
+                $self->_error( "$spec_error for ':key'" );
             }
 
 
@@ -476,7 +497,7 @@ sub check_list {
         } elsif ($spec->{':key'}) {
             $self->check_map($spec,$value);
         } else {
-            $self->_error( "Unknown value type, '$value', found in list structure" );
+          $self->_error( "$spec_error associated with '$self->{stack}[-2]'" );
         }
         pop @{$self->{stack}};
     }
@@ -557,6 +578,8 @@ sub urlspec {
     return 0;
 }
 
+sub anything { return 1 }
+
 sub string {
     my ($self,$key,$value) = @_;
     if(defined $value) {
@@ -770,7 +793,7 @@ CPAN::Meta::Validator - validate CPAN distribution metadata structures
 
 =head1 VERSION
 
-version 2.101091
+version 2.101110
 
 =head1 SYNOPSIS
 
